﻿using Microsoft.SharePoint;
using Microsoft.SharePoint.Utilities;
using Microsoft.SharePoint.WebControls;
using System;

namespace zSiS.CopyItem.Layouts.zSiS
{
    public partial class CopyItem : LayoutsPageBase
    {
        #region Constants

        const string LISTNAMEPAR = "ListId";
        const string ITEMIDPAR = "ItemId";
        const string SOURCEFILEURL = "SourceFileUrl";
        const string NOPERMISSION = "You do not have permission to copy the files in this folder";
        const string ZSISCOPYITE = "zSiS.CopyItem";

        #endregion

        #region ViewState Properties

        private Guid? ListID
        {
            get
            {
                if (ViewState[LISTNAMEPAR] == null)
                    return null;
                else
                    return new Guid(ViewState[LISTNAMEPAR].ToString());
            }
            set
            {
                ViewState[LISTNAMEPAR] = value;
            }
        }

        private int ItemId
        {
            get
            {
                return int.Parse(ViewState[ITEMIDPAR].ToString());
            }
            set
            {
                ViewState[ITEMIDPAR] = value;
            }
        }

        private string SourceFileUrl
        {
            get
            {
                return ViewState[SOURCEFILEURL].ToString();
            }
            set
            {
                ViewState[SOURCEFILEURL] = value;
            }
        }

        #endregion

        #region Other Properties

        private SPUser CurrentUser
        {
            get
            {
                return SPContext.Current.Web.EnsureUser(SPContext.Current.Web.CurrentUser.LoginName);
            }
        }

        #endregion

        #region Form Events

        protected void Page_Load(object sender, EventArgs e)
        {
            try
            {
                if (!Page.IsPostBack)
                {
                    //Checking Parameter ListId
                    if (Request.Params[LISTNAMEPAR] == null)
                        SPUtility.TransferToErrorPage("Parameter ListId does not exist.");

                    //Checking Parameter ItemId
                    if (Request.Params[ITEMIDPAR] == null)
                        SPUtility.TransferToErrorPage("Parameter ItemId does not exist.");

                    //Checking if Parameter ItemId is a valid int
                    int itemId = -1;
                    if (!int.TryParse(Request.Params[ITEMIDPAR], out itemId))
                        SPUtility.TransferToErrorPage("Parameter ItemId is not a valid int.");

                    //Setting properties ItemId and ListID
                    ItemId = itemId;
                    ListID = new Guid(Request.Params[LISTNAMEPAR]);

                    //Getting current web
                    SPWeb web = SPContext.Current.Web;

                    //Getting source List
                    SPList sourceList = web.Lists[(Guid)ListID];
                    if (sourceList == null)
                        SPUtility.TransferToErrorPage("List " + ListID + " does not exist in web site " + web.Url);

                    //Getting source item
                    SPListItem sourceItem = null;

                    try
                    {
                        sourceItem = sourceList.GetItemById(ItemId);
                    }
                    catch (Exception)
                    {
                        SPUtility.TransferToErrorPage("Item " + ItemId + " does not exist in the list " + ListID + " in web site " + web.Url);
                    }

                    //Getting source file
                    SourceFileUrl = sourceItem.Url;

                    //Loading web lists
                    populateLists(web);

                    //Collpase all nodes
                    tvBrowse.CollapseAll();

                    //Popupulating source file information section
                    populateSourceInfo(sourceItem);

                }

                //Enabling Ok button only if lblFullUrl contains a text
                cmdOk.Enabled = !string.IsNullOrEmpty(lblFullUrl.Text);


            }
            //Catching ThreadAbortException to prevent error on TransferToErrorPage
            catch (System.Threading.ThreadAbortException)
            {
            }
            catch (Exception ex)
            {
                try
                {
                    LoggingService.LogError(ZSISCOPYITE, ex.ToString());
                    SPUtility.TransferToErrorPage("Error occurred: " + ex.Message);
                }
                catch (System.Threading.ThreadAbortException)
                {
                }
                throw;
            }
        }


        protected void tvBrowse_SelectedNodeChanged(object sender, EventArgs e)
        {
            //Setting lblFullUrl text with the full URL
            lblFullUrl.Text = tvBrowse.SelectedNode.Value + "/" + txtFileName.Text;


            //Enabling Ok button only if lblFullUrl contains a text
            cmdOk.Enabled = !string.IsNullOrEmpty(lblFullUrl.Text);
        }

        protected void txtFileName_TextChanged(object sender, EventArgs e)
        {
            //Checking the file name
            if (string.IsNullOrEmpty(txtFileName.Text))
            {
                lblErrore.Text = "Select a valid file name";
                return;
            }

            //Setting lblFullUrl text with the full URL
            if (tvBrowse.SelectedNode != null)
                lblFullUrl.Text = tvBrowse.SelectedNode.Value + "/" + txtFileName.Text;
        }

        protected void cmdOk_Click(object sender, EventArgs e)
        {
            if (Page.IsValid)
            {
                lblErrore.Text = string.Empty;

                //Getting current web
                SPWeb web = SPContext.Current.Web;

                //Getting current source file
                SPFile sourceFile = web.GetFile(SourceFileUrl);

                //Performing integraty controls
                if (tvBrowse.SelectedNode == null)
                {
                    lblErrore.Text = "Select a valid destination folder.";
                    return;
                }

                if (string.IsNullOrEmpty(lblFullUrl.Text))
                {
                    lblErrore.Text = "Destination path is not valid.";
                    return;
                }

                if (!sourceFile.Exists)
                {
                    lblErrore.Text = "Source file does not exist.";
                    return;
                }

                if (string.IsNullOrEmpty(txtFileName.Text))
                {
                    lblErrore.Text = "Select a valid file name";
                    return;
                }

                try
                {
                    //Copying the file
                    sourceFile.CopyTo(lblFullUrl.Text);

                    //Transger to success page
                    SPUtility.TransferToSuccessPage("File " + lblSourceFileName.Text + " has been copied to the destination url " + lblFullUrl.Text);

                }
                catch (Exception ex)
                {
                    lblErrore.Text = ex.Message;
                    LoggingService.LogError(ZSISCOPYITE, ex.ToString());
                }
            }
        }

        #endregion

        #region Private functions

        private void populateSourceInfo(SPListItem sourceItem)
        {
            txtFileName.Text = System.IO.Path.GetFileName(SourceFileUrl);
            lblSourceFileName.Text = txtFileName.Text;
            lblSourceFileUrl.Text = SourceFileUrl;
            lblSourceTitle.Text = sourceItem.Title;
            lblSourceDocLib.Text = SPContext.Current.Web.Lists[(Guid)ListID].Title;
        }

        private void populateLists(SPWeb web)
        {
            foreach (SPList myList in web.Lists)
            {
                if (myList.BaseType == SPBaseType.DocumentLibrary)
                {
                    System.Web.UI.WebControls.TreeNode myNode = new System.Web.UI.WebControls.TreeNode(myList.Title, myList.RootFolder.Url, myList.ImageUrl);

                    if (!myList.DoesUserHavePermissions(CurrentUser, SPBasePermissions.AddListItems))
                    {
                        myNode.SelectAction = System.Web.UI.WebControls.TreeNodeSelectAction.None;
                        myNode.ToolTip = NOPERMISSION;
                    }

                    tvBrowse.Nodes.Add(myNode);

                    populateFolders(ref myNode, myList.RootFolder);

                }
            }
        }

        private void populateFolders(ref System.Web.UI.WebControls.TreeNode myNode, SPFolder myFolder)
        {
            const string folderImageUrl = "/_layouts/15/images/folder.gif";
            const string docSetImageUrl = "/_layouts/15/images/icdocset.gif";

            foreach (SPFolder mySubFolder in myFolder.SubFolders)
            {
                System.Web.UI.WebControls.TreeNode myFolderNode = new System.Web.UI.WebControls.TreeNode(mySubFolder.Name, mySubFolder.Url, folderImageUrl);
                if (mySubFolder.Item != null)
                {
                    if (!mySubFolder.Item.DoesUserHavePermissions(CurrentUser, SPBasePermissions.AddListItems))
                    {
                        myFolderNode.SelectAction = System.Web.UI.WebControls.TreeNodeSelectAction.None;
                        myFolderNode.ToolTip = NOPERMISSION;
                    }

                    if (mySubFolder.Item.ContentType.Id.IsChildOf(SPBuiltInContentTypeId.DocumentSet))
                        myFolderNode.ImageUrl = docSetImageUrl;

                    myNode.ChildNodes.Add(myFolderNode);
                    populateFolders(ref myFolderNode, mySubFolder);
                }
            }
        }

        #endregion


    }
}
